/*---------------------------------------------------------------------------*/

/*!  \file  kww_freq.c
     \brief  calculate the KWW relaxation function in the frequency domain.  */

/*---------------------------------------------------------------------------*/

/* Copyright (C) 2011 Takashi Uneyama  */

/* This program is free software; you can redistribute it and/or modify
   it under the terms of the GNU General Public License as published by
   the Free Software Foundation; either version 2, or (at your option)
   any later version.

   This program is distributed in the hope that it will be useful,
   but WITHOUT ANY WARRANTY; without even the implied warranty of
   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
   GNU General Public License for more details.

   You should have received a copy of the GNU General Public License
   along with this program; if not, write to the Free Software
   Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307,
   USA.  */

/*---------------------------------------------------------------------------*/
#include <stdio.h>
#include <stdlib.h>
#include <math.h>

/*---------------------------------------------------------------------------*/
#define EPSILON (1.0e-15)

/*---------------------------------------------------------------------------*/
void intdeo(double (*f)(double), double a, double omega,
            double eps, double *i, double *err);
void intdei(double (*f)(double), double a, double eps, 
            double *i, double *err);

/*---------------------------------------------------------------------------*/
static void get_parameters(int argc,char **argv,double *beta,double *omega_min,
                           double *omega_max,int *n_omega,int *type);
static double f_sin(double x);
static double f_cos(double x);

/*---------------------------------------------------------------------------*/
double beta,omega;

static char *helpstring=
"\
Usage: %s beta omega_min omega_max n_omega type\n\
    beta       the exponent for the KWW relaxation function\n\
    omega_min  minimum angular frequency\n\
    omega_max  maximum angular frequency\n\
    n_omega    number of divisions for the angular frequency\n\
    type       0: dielectric relaxation, 1: viscoelastic modulus\n\
";

/*---------------------------------------------------------------------------*/
int main(int argc,char **argv)
{
    int j;
    double omega_min,omega_max;
    int n_omega,type;
    double f0,f1,f2,err;
    double ln_omega_min,ln_omega_max,delta_ln_omega;

    get_parameters(argc,argv,&beta,&omega_min,&omega_max,&n_omega,&type);

    ln_omega_min = log(omega_min);
    ln_omega_max = log(omega_max);
    delta_ln_omega = (ln_omega_max - ln_omega_min) / n_omega;

    if(type == 0)
    {
        omega = 0.0;
        intdei(f_cos,0.0,EPSILON,&f0,&err);
    }

    for(j = 0;j <= n_omega;j++)
    {
        omega = exp(ln_omega_min + j * delta_ln_omega);

        intdeo(f_cos,0.0,omega,EPSILON,&f1,&err);
        intdeo(f_sin,0.0,omega,EPSILON,&f2,&err);

        if(type == 0)
            printf("%g %g %g\n",omega,f1 / f0,f2 / f0);
        else
            printf("%g %g %g\n",omega,omega * f2,omega * f1);
    }

    return 0;
}

/*---------------------------------------------------------------------------*/
void get_parameters(int argc,char **argv,double *beta,double *omega_min,
                    double *omega_max,int *n_omega,int *type)
{
    if(argc != 6)
    {
        fprintf(stderr,helpstring,argv[0]);
        exit(EXIT_FAILURE);
    }

    sscanf(argv[1],"%lf",beta);
    sscanf(argv[2],"%lf",omega_min);
    sscanf(argv[3],"%lf",omega_max);
    sscanf(argv[4],"%d",n_omega);
    sscanf(argv[5],"%d",type);

    if((*beta <= 0.0) || (*beta > 1.0))
    {
        fprintf(stderr,"error: beta = %f.\n`beta' should be in the range (0,1].\n",
                *beta);
        exit(EXIT_FAILURE);
    }

    if((*omega_min <= 0.0) || (*omega_max <= 0.0))
    {
        fprintf(stderr,"error: omega_min = %g, omega_max = %g.\n`omega_min' and 'omega_max' should be positive.\n",
                *omega_min,*omega_max);
        exit(EXIT_FAILURE);
    }

    if(*n_omega < 1)
    {
        fprintf(stderr,"error: n_omega = %d.\n`n_omega' should be larger than 0.\n",
                *n_omega);
        exit(EXIT_FAILURE);
    }

    if(!((*type == 0) || (*type == 1)))
    {
        fprintf(stderr,"error: type = %d.\n`type' should be either 0 or 1.\n",
                *type);
        exit(EXIT_FAILURE);
    }
}

/*---------------------------------------------------------------------------*/
double f_sin(double t)
{
    return (exp(-pow(t,beta)) * sin(omega * t));
}

/*---------------------------------------------------------------------------*/
double f_cos(double t)
{
    return (exp(-pow(t,beta)) * cos(omega * t));
}

/*---------------------------------------------------------------------------*/
